#ifndef ROOTS_H
#define ROOTS_H

extern int roots(unsigned char *, unsigned char *, int *, unsigned char *);
extern int roots_same(unsigned char *, unsigned char *);
extern int roots_init(char *);

#endif
